/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file video-player.cpp
/// @brief Example program for QVision library. Simple example of image input/output from a video file, to a canvas window
///
/// This program will receive a path or url to a video file, and then will read each frame and show each YUV channel in a canvas window.
/// To use the program, simply compile, and execute the binary:
///
///	./video-player --URL=<path or url to video>
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>

class PlayerWorker: public QVWorker
	{
	public:
		PlayerWorker(QString name): QVWorker(name)
			{
			addProperty< QVImage<uChar,1> >("Input image Y", inputFlag);
			addProperty< QVImage<uChar,1> >("Input image U", inputFlag);
			addProperty< QVImage<uChar,1> >("Input image V", inputFlag);

			addProperty< QVImage<uChar,1> >("Output image Y", outputFlag);
			addProperty< QVImage<uChar,1> >("Output image U", outputFlag);
			addProperty< QVImage<uChar,1> >("Output image V", outputFlag);
			}

		void worker()
			{
			QVImage<uChar, 1>	imageY = getPropertyValue< QVImage<uChar,1> >("Input image Y"),
						imageU = getPropertyValue< QVImage<uChar,1> >("Input image U"),
						imageV = getPropertyValue< QVImage<uChar,1> >("Input image V");

			setPropertyValue< QVImage<uChar,1> >("Output image Y", imageY);
			setPropertyValue< QVImage<uChar,1> >("Output image U", imageU);
			setPropertyValue< QVImage<uChar,1> >("Output image V", imageV);
			}
	};

int main(int argc, char *argv[])
	{
	
	QVApplication app(argc, argv,
		"Example program for QVision library. Shows the content of a video file."
		);

	#ifdef QT_NO_DEBUG
		std::cout << "RELEASE" << std::endl;
	#endif

	#ifndef QT_NO_DEBUG
		std::cout << "DEBUG" << std::endl;
	#endif

	QVMPlayerCamera camera("Video");
	PlayerWorker worker("Video player worker");
	QVisionInterface interface;

	camera.link(&worker,"Input image Y", "Input image U", "Input image V");
	interface.setVisible(&worker,"Output image Y");
	interface.setVisible(&worker,"Output image U");
	interface.setVisible(&worker,"Output image V");

	return app.exec();
	}
