/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcontour.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvdta/qvcontour.h>

namespace qvdta
{
// Direction-número		Y
//	NE-7	N-0	NW-1	|
//	E-6	*	W-2	v
//	SE-5	S-4	SW-3
// X -->
//				N	NO	O	SO	S	SE	E	NE
const char	coorX[8] = {	0,	1,	1,	1,	0,	-1,	-1,	-1 };
const char	coorY[8] = {	-1,	-1,	0,	1,	1,	1,	0,	-1 };

// Auxiliar function for border extraction. It gets a border point, and the direction where there is one of the outside of the connected-set pixels.
QVPolyline getContourThresholdFromBorderPoint(const QVImage<uChar> &image, const int startPointX, const int startPointY, const uChar threshold)
	{
	//std::cout << "xx"<< std::endl;
	QVPolyline lista;

	//std::cout << "xx 1"<< std::endl;
	lista.loop = true;
	lista.append(QPoint(startPointX, startPointY));

	QVIMAGE_INIT_READ(uChar,image);
	QRect roi = image.getROI();

	//std::cout << "xx 2"<< std::endl;
	Q_ASSERT_X(roi.contains(startPointX, startPointY), "getContourThresholdFromBorderPoint", "start point out of image ROI");
	Q_ASSERT_X(QVIMAGE_PIXEL(image, startPointX, startPointY, 0) >= threshold, "getContourThresholdFromBorderPoint", "start point is not contained in a connected set");

	//std::cout << "xx 3" << std::endl;

	// Comprobamos tanto que no tenemos un píxel interior, como que no tenemos un píxel solitario.
	// De paso buscamos una dirección en la que haya un píxel vacío (que no pertenezca a un conjunto).
	uChar searchDir = 128, numOuterPixels = 0;
	for (int i = 0; i<8; i++)
		{
		int x =  startPointX +coorX[i], y =  startPointY +coorY[i];
		if (!roi.contains(x, y))
			{
			numOuterPixels++;
			searchDir = i;
			}
		else if (QVIMAGE_PIXEL(image, x, y,0) < threshold)
			{
			numOuterPixels++;
			searchDir = i;
			}
		}

	// Caso de que se haya proporcionado un píxel interior, assert
	Q_ASSERT_X(searchDir < 8, "getContourThresholdFromBorderPoint", "start point is inside the set, not in the border");

	// Caso de que se haya proporcionado un píxel solitario, devolvermos ese pixeĺ sólo:
	if (numOuterPixels == 8)
		return lista;

	//std::cout << "xx 4" << std::endl;

	// Recorremos cada punto del contorno, insertándolo en la lista de puntos.
	int sumSearchDir = 0, actualPointX = startPointX, actualPointY = startPointY;
	while (true)
		{
		// We search for the next point belonging to the contour.
		uChar d;
		int	nextPointX, nextPointY;
		for (d = 0; d < 8; d++)
			{
			searchDir = (searchDir+1)%8;
			nextPointX = actualPointX + coorX[searchDir];
			nextPointY = actualPointY + coorY[searchDir];
			if (roi.contains(nextPointX, nextPointY))
				if ( (QVIMAGE_PIXEL(image, nextPointX, nextPointY,0) >= threshold) )
					break;
			}

		sumSearchDir += d - 3;

		actualPointX = nextPointX;
		actualPointY = nextPointY;

		if ( QVIMAGE_PIXEL(image, actualPointX, actualPointY,0) < threshold )
			break;

		if ( startPointX == actualPointX && startPointY == actualPointY)
			break;

		lista.append(QPoint(actualPointX, actualPointY));
		//std::cout << " 4" << actualPointX << ", " << actualPointY << std::endl;
		searchDir = searchDir + 4;
		}

	//std::cout << " 4" << std::endl;

	lista.direction = (sumSearchDir >= 0);
	lista.dir = sumSearchDir;

	//std::cout << " end" << std::endl;
	return lista;
	}

QVPolyline getContourThreshold(const QVImage<uChar> &image, const QPoint startPoint, const uChar threshold)
	{
	// TODO: esto tiene que arreglarse.
	//QVIMAGE_INIT_READ(uChar,image);
	//QRect roi = image.getROI();

	int	startPointX = startPoint.x(), startPointY = startPoint.y();

	//Q_ASSERT_X(roi.contains(startPointX, startPointY), "getContourThreshold", "start point out of image ROI");
	//Q_ASSERT_X(QVIMAGE_PIXEL(image, startPointX, startPointY, 0) >= threshold, "getContourThreshold", "start point is not contained in a connected set");

	// Buscamos un punto alrededor del inicial, en sentido de las agujas de reloj, que NO
	// pertenezca al contorno. (debe de existir, si el que use la clase se asegura de
	// crearla con un punto adecuado).
	/*uChar searchDir;
	for (searchDir = 0; searchDir<8; searchDir++)
		{
		int	startX = startPoint.x() + coorX[searchDir],
			startY = startPoint.y() + coorY[searchDir];

		if (!roi.contains(startX, startY))
			break;

		if ( !(QVIMAGE_PIXEL(image, startX, startY,0) >= threshold) )
			break;
		}

	searchDir = (searchDir+4)%8;*/

	return getContourThresholdFromBorderPoint(image, startPointX, startPointY, threshold);
	}

QList<QVPolyline> getContoursThreshold(const QVImage <uChar> &image, const uChar threshold)
	{
	//std::cout << "in" << std::endl;
	qDebug() << "getPolylinesThreshold()";
	QVImage<uChar> mask(image.getCols()+1, image.getRows()+1);
	qvipp::Set(mask,0);
	//std::cout << "in 1" << std::endl;
	QVIMAGE_INIT_READ(uChar,image);
	QVIMAGE_INIT_WRITE(uChar,mask);

	QRect roi = image.getROI();

	QList<QVPolyline> polylineList;
	//std::cout << "in 2" << std::endl;
	// Recorremos la imagen buscando píxeles que correspondan a algún grupo
	for (int row = roi.y(); row < roi.y() + roi.height(); row++)
		for (int col = roi.x(); col < roi.y() + roi.width(); col++)
			{
			// En cada paso siempre que encontremos un píxel que pertenezca a algún conjunto conexo, pertenecerá a su contorno.
			if (QVIMAGE_PIXEL(image, col, row,0) >= threshold)
				{
				//std::cout << "\tloop start " << col << ", " << row << std::endl;

				// si no están marcados, obtenemos el contorno al que pertenecen.
				if ( !QVIMAGE_PIXEL(mask, col, row,0) )
					{
					//std::cout << "\tZ" << std::endl;
					QVPolyline lista = getContourThresholdFromBorderPoint(image, col, row, threshold);
					//std::cout << "\tA" << std::endl;
					draw(mask, lista, true, true);
					//std::cout << "\tB" << std::endl;
					polylineList.append(lista);
					//std::cout << "\tC" << std::endl;
					}
				//std::cout << "\tloop 1" << std::endl;

				// Nos aseguramos de que el siguiente píxel pertenezca al contorno.
				while (roi.contains(col+1, row))
					{
					if ( QVIMAGE_PIXEL(image, col+1, row,0) < threshold )
						break;
					col++;
					}
				//std::cout << "\tloop 2" << std::endl;

				// esto es por si tenemos un contorno interno, puede ser que no se procese correctamente. Nos aseguramos.
				if ( !QVIMAGE_PIXEL(mask, col, row,0) )
					{
					QVPolyline lista = getContourThresholdFromBorderPoint(image, col, row, threshold);
					draw(mask, lista, true, true);
					polylineList.append(lista);
					}
				//std::cout << "\tloop end" << std::endl;
				}

			}
	//std::cout << "in 3" << std::endl;
	qDebug() << "getPolylinesThreshold():"<< polylineList.size() << "contours obtained";
	qDebug() << "getPolylinesThreshold() <~ return";
	return polylineList;
	}
}
