/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamsinspectorwidget.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVPARAMSINSPECTORWIDGET_H
#define QVPARAMSINSPECTORWIDGET_H

#include <QDebug>
#include <QtGui/QWidget>
#include <QtGui/QGroupBox>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <QtGui/QSpacerItem>
#include <QMetaObject>
#include <QMetaMethod>

#include <qvcore/qvpropertyholder.h>
#include <qvgui/qvparamwidget.h>

/// \cond Not documented by now
/// @file qvparamsinspector.h
/// @brief QVParamsInspector interface file.
/// @author PARP Group (http://perception.inf.um.es), Univ. of Murcia (Spain)

/// @class QVParamsInspectorWidget qvparamsinspectorwidget.h qvgui/qvparamsinspectorwidget.h
/// @brief QVParamsInspectorWidget class.
///
/// This class implements a QWidget that shows controls to preview and modify the state of double, int and boolean
/// properties in a QVPropertyHolder object, in execution time.
/// @ingroup qvgui
class QVParamsInspectorWidget: public QWidget
{
	Q_OBJECT
public:
	QVBoxLayout *vboxLayout;

	/// @brief Constructs an empty QVParamsInspectorWidget object.
	/// @param name identifier for the object to be created.
	QVParamsInspectorWidget(QVPropertyHolder * object = NULL, QWidget *parent = 0);

private:
	/// @brief Adds to the widget new buttons and sliders to control and preview in real time integer, double and
	/// boolean typed property values from a new QVPropertyHolder object. Any other property from previously added
	/// QVPropertyHolder will be kept shown in the widget
	/// @param propertyHolder The new QVPropertyHolder object which integer, double and boolean
	/// properties will be depicted in the widget as new buttons and sliders
	void add(QVPropertyHolder *propertyHolder);

private:
	void add(QVIntParamWidget *widget);
	void add(QVBoolParamWidget *widget);
	void add(QVDoubleParamWidget *widget);
};
/// \endcond Not documented by now
#endif

